@@ -16,7 +16,6 @@ from .utilities import (
     normalize_name,
     get_installed_distributions,
     is_requirement_string,
-    reinstall_editable,
 )
 
 
@@ -64,7 +63,7 @@ def _update_requirement_specifiers(
     for specifier in requirement.specifier:  # type: ignore
         # Only update requirement to match our installed version
         # if the requirement is *inclusive*
-        if "=" in specifier.operator:
+        if ("=" in specifier.operator) and ("!" not in specifier.operator):
             specifier_version: Union[Version, LegacyVersion] = parse_version(
                 specifier.version
             )
@@ -203,15 +202,16 @@ def get_updated_setup_cfg(
         extra_requirements_string: str
         extra_requirements: List[str]
         for extra_name, extra_requirements_string in extras_require.items():
-            extra_requirements = list(
-                map(
-                    get_updated_requirement_string,
-                    extra_requirements_string.split("\n"),
+            if extra_name != all_extra_name:
+                extra_requirements = list(
+                    map(
+                        get_updated_requirement_string,
+                        extra_requirements_string.split("\n"),
+                    )
                 )
-            )
-            if all_extra_name:
-                all_extra_requirements += extra_requirements
-            extras_require[extra_name] = "\n".join(extra_requirements)
+                if all_extra_name:
+                    all_extra_requirements += extra_requirements
+                extras_require[extra_name] = "\n".join(extra_requirements)
         # If a name was specified for an all-encompasing extra,
         # we de-duplicate and update or create that extra
         if all_extra_name:
@@ -248,19 +248,24 @@ def get_updated_tox_ini(data: str, ignore: Iterable[str] = ()) -> str:
     parser: ConfigParser = ConfigParser()
     parser.read_string(data)
 
-    def update_section(section_name: str) -> None:
-        if parser.has_option(section_name, "deps"):
+    def update_section_options(section_name: str, option_name: str) -> None:
+        if parser.has_option(section_name, option_name):
             parser.set(
                 section_name,
-                "deps",
+                option_name,
                 "\n".join(
                     map(
                         get_updated_requirement_string,
-                        parser.get(section_name, "deps").split("\n"),
+                        parser.get(section_name, option_name).split("\n"),
                     )
                 ),
             )
 
+    def update_section(section_name: str) -> None:
+        update_section_options(section_name, "deps")
+        if section_name == "tox":
+            update_section_options(section_name, "requires")
+
     # Update
     list(map(update_section, parser.sections()))
     # Return as a string
@@ -355,23 +360,11 @@ def update(
     """
     if isinstance(paths, str):
         paths = (paths,)
-    path_directories: Iterable[str] = map(
-        os.path.abspath,  # type: ignore
-        map(os.path.dirname, paths),  # type: ignore
-    )
-    # Reinstall all editable distributions, excluding those wherein
-    # the files being updated live (since there may be invalid requirement
-    # specifiers in these files until the updates are performed)
-    reinstall_editable(exclude_locations=path_directories, echo=verbose)
 
     def update_(path: str) -> None:
         _update(path, ignore=ignore, all_extra_name=all_extra_name)
 
     list(map(update_, paths))
-    # Reinstall any editable distributions which live in the same
-    # directory as the files being updated (and therefore typically belonging
-    # to the same project)
-    reinstall_editable(include_locations=path_directories, echo=verbose)
 
 
 def main() -> None:
