@@ -2,12 +2,24 @@ import sys
 import os
 import tomli
 import pkg_resources
+import importlib_metadata
 from warnings import warn
 from pipes import quote
 from configparser import ConfigParser, SectionProxy
 from enum import Enum, auto
 from itertools import chain
-from typing import Dict, Iterable, Set, Tuple, List, IO, Union, Callable, Any
+from typing import (
+    Optional,
+    Dict,
+    Iterable,
+    Set,
+    Tuple,
+    List,
+    IO,
+    Union,
+    Callable,
+    Any,
+)
 from packaging.utils import canonicalize_name
 from packaging.requirements import InvalidRequirement, Requirement
 from more_itertools import unique_everseen
@@ -275,17 +287,20 @@ def _iter_editable_distributions(
     )
 
 
-def _reinstall_distribution(
+def _reinstall_editable_distribution(
     distribution: pkg_resources.Distribution, echo: bool = False
-) -> None:
-    _reinstall_location(distribution.location, echo=echo)
+) -> bool:
+    return _reinstall_location(distribution.location, echo=echo)
 
 
-def _reinstall_location(location: str, echo: bool = False) -> bool:
+def _reinstall_location(
+    location: str, user: bool = True, echo: bool = False
+) -> bool:
+    user_flag = "--user " if user else ""
     try:
         run(
             (
-                f"{quote(sys.executable)} -m pip install --no-deps "
+                f"{quote(sys.executable)} -m pip install {user_flag}--no-deps "
                 f"-e {quote(location)}"
             ),
             echo=echo,
@@ -293,9 +308,12 @@ def _reinstall_location(location: str, echo: bool = False) -> bool:
         _reinstalled_locations.add(os.path.abspath(location))
         return True
     except OSError:
-        # If an error code is returned, we just assume package metadata is
-        # up-to-date
-        return False
+        if user:
+            return _reinstall_location(location, user=False, echo=echo)
+        else:
+            # If an error code is returned, we just assume package metadata is
+            # up-to-date
+            return False
 
 
 def reinstall_location(location: str, echo: bool = False) -> str:
@@ -354,7 +372,7 @@ def reinstall_editable(
     def reinstall_distribution_(
         distribution: pkg_resources.Distribution,
     ) -> None:
-        _reinstall_distribution(distribution, echo=echo)
+        _reinstall_editable_distribution(distribution, echo=echo)
 
     list(
         map(
@@ -411,27 +429,20 @@ def get_setup_distribution_name(path: str) -> str:
     )
 
 
-def _setup_dist_egg_info(location: str) -> None:
-    setup_py_path: str = os.path.join(location, "setup.py")
+def _setup_egg_info(location: str) -> None:
     # If there is no setup.py file, we can't update egg info
-    if not os.path.isfile(setup_py_path):
+    if not os.path.isfile(os.path.join(location, "setup.py")):
         return
     current_directory: str = os.curdir
     os.chdir(location)
-    try:
-        run(
-            (
-                f"{quote(sys.executable)} {quote(setup_py_path)} -q "
-                "dist_info egg_info"
-            ),
-            echo=False,
-        )
-    except OSError:
-        pass
+    run(
+        f"{quote(sys.executable)} setup.py -q egg_info",
+        echo=False,
+    )
     os.chdir(current_directory)
 
 
-def setup_dist_egg_info(directory: str) -> None:
+def setup_egg_info(directory: str) -> None:
     """
     Refresh dist-info and egg-info for the editable package installed in
     `directory`
@@ -439,7 +450,7 @@ def setup_dist_egg_info(directory: str) -> None:
     directory = os.path.abspath(directory)
     if not os.path.isdir(directory):
         directory = os.path.dirname(directory)
-    return _setup_dist_egg_info(directory)
+    return _setup_egg_info(directory)
 
 
 def get_setup_distribution_requirements(
@@ -459,7 +470,7 @@ def _get_location_distribution_name(location: str) -> str:
     Get a distribution name based on an installation location, or return
     an empty string if no distribution can be found
     """
-    _setup_dist_egg_info(location)
+    _setup_egg_info(location)
     location = os.path.abspath(location)
 
     def _is_in_location(
@@ -568,6 +579,96 @@ def _get_pkg_requirement_name(requirement: pkg_resources.Requirement) -> str:
     return normalize_name(requirement.project_name)
 
 
+def install_requirement(
+    requirement: Union[str, Requirement, pkg_resources.Requirement],
+    echo: bool = True,
+) -> None:
+    """
+    Install a requirement
+
+    Parameters:
+
+    - requirement (str)
+    - echo (bool) = True: If `True` (default), the `pip install`
+      commands will be echoed to `sys.stdout`
+    """
+    requirement_string: str = (
+        requirement if isinstance(requirement, str) else str(requirement)
+    )
+    if isinstance(requirement, str):
+        requirement = Requirement(requirement)
+    # Get the distribution name
+    name: str = normalize_name(
+        requirement.name
+        if isinstance(requirement, Requirement)
+        else requirement.project_name
+    )
+    distribution: Optional[pkg_resources.Distribution] = None
+    editable: bool = False
+    try:
+        distribution = get_distribution(name)
+        editable = _distribution_is_editable(distribution)
+    except KeyError:
+        pass
+    # If the requirement is installed and editable, re-install from
+    # the editable location
+    if distribution and editable:
+        # Refresh metadata
+        setup_egg_info(distribution.location)
+        # Assemble a requirement specifier for the editable install
+        requirement_string = distribution.location
+        if requirement.extras:
+            requirement_string = (
+                f"{requirement_string}" f"[{','.join(requirement.extras)}]"
+            )
+    # First attempt a user install, then a system install
+    try:
+        run(
+            (
+                f"{quote(sys.executable)} -m pip install --user "
+                f"{quote(requirement_string)}"
+            ),
+            echo=echo,
+        )
+    except OSError:
+        # If a user installation failed, attempt a system installation
+        run(
+            (
+                f"{quote(sys.executable)} -m pip install "
+                f"{quote(requirement_string)}"
+            ),
+            echo=echo,
+        )
+    # Refresh the metadata
+    if distribution and editable:
+        setup_egg_info(distribution.location)
+    else:
+        refresh_working_set()
+
+
+def _get_pkg_requirement_distribution(
+    requirement: pkg_resources.Requirement, name: str, reinstall: bool = True
+) -> Optional[pkg_resources.Distribution]:
+    try:
+        return get_installed_distributions()[name]
+    except KeyError:
+        if not reinstall:
+            raise
+        warn(
+            f'The required distribution "{name}" was not installed, '
+            "attempting to install it now..."
+        )
+        try:
+            # Attempt to install the requirement...
+            install_requirement(requirement)
+        except OSError:
+            return None
+        refresh_working_set()
+        return _get_pkg_requirement_distribution(
+            requirement, name, reinstall=False
+        )
+
+
 def _iter_requirement_names(
     requirement: pkg_resources.Requirement,
     exclude: Set[str],
@@ -577,26 +678,14 @@ def _iter_requirement_names(
     extras: Set[str] = set(map(normalize_name, requirement.extras))
     if name in exclude:
         return ()
-    distribution: pkg_resources.Distribution
-    try:
-        distribution = get_installed_distributions()[name]
-    except KeyError:
-        warn(
-            'The distribution "{name}" was not installed, '
-            "attempting to install it now..."
-        )
-        run(
-            (
-                f"{quote(sys.executable)} -m pip install "
-                f"{quote(str(requirement))}"
-            ),
-            echo=True,
-        )
-        refresh_working_set()
-        return _iter_requirement_names(requirement, exclude, recursive)
+    distribution: Optional[
+        pkg_resources.Distribution
+    ] = _get_pkg_requirement_distribution(requirement, name)
+    if distribution is None:
+        return ()
     # Ensure requirements are up-to-date
     if _distribution_is_editable(distribution):
-        _setup_dist_egg_info(distribution.location)
+        _setup_egg_info(distribution.location)
     requirements: List[pkg_resources.Requirement] = distribution.requires(
         extras=tuple(sorted(extras))
     )
@@ -605,11 +694,17 @@ def _iter_requirement_names(
         requirement_: pkg_resources.Requirement,
     ) -> Iterable[str]:
         return _iter_requirement_names(
-            requirement_, exclude=exclude, recursive=recursive
+            requirement_,
+            exclude - {_get_pkg_requirement_name(requirement_)},
+            recursive,
         )
 
     def not_excluded(name: str) -> bool:
-        return name not in exclude
+        if name not in exclude:
+            # Add this to the exclusions
+            exclude.add(name)
+            return True
+        return False
 
     requirement_names: Iterable[str] = filter(
         not_excluded, map(_get_pkg_requirement_name, requirements)
@@ -686,3 +781,18 @@ def get_requirements_required_distribution_names(
             key=lambda name: name.lower(),
         )
     )
+
+
+def iter_distribution_location_file_paths(location: str) -> Iterable[str]:
+    location = os.path.abspath(location)
+    name: str = get_setup_distribution_name(location)
+    setup_egg_info(location)
+    metadata_path: str = os.path.join(
+        location, f"{pkg_resources.to_filename(name)}.egg-info"
+    )
+    distribution: importlib_metadata.Distribution = (
+        importlib_metadata.Distribution.at(metadata_path)
+    )
+    if not distribution.files:
+        raise RuntimeError(f"No metadata found at {metadata_path}")
+    return map(os.path.abspath, distribution.files)
